@@ -43,8 +43,6 @@ struct GvcStreamStatusIconPrivate
         guint           current_icon;
         char           *display_name;
         gboolean        thaw;
-        GdkDevice      *grabbed_pointer;
-        GdkDevice      *grabbed_keyboard;
 };
 
 enum
@@ -55,8 +53,6 @@ enum
         PROP_ICON_NAMES,
 };
 
-static void     gvc_stream_status_icon_class_init (GvcStreamStatusIconClass *klass);
-static void     gvc_stream_status_icon_init       (GvcStreamStatusIcon      *stream_status_icon);
 static void     gvc_stream_status_icon_finalize   (GObject                  *object);
 
 G_DEFINE_TYPE (GvcStreamStatusIcon, gvc_stream_status_icon, GTK_TYPE_STATUS_ICON)
@@ -101,15 +97,14 @@ static void
 ungrab (GvcStreamStatusIcon *icon,
         guint                time)
 {
-	/* ungrab focus */
-	if (icon->priv->grabbed_pointer != NULL) {
-		gdk_device_ungrab (icon->priv->grabbed_pointer, time);
-		g_clear_object (&icon->priv->grabbed_pointer);
-	}
-	if (icon->priv->grabbed_keyboard != NULL) {
-		gdk_device_ungrab (icon->priv->grabbed_keyboard, time);
-		g_clear_object (&icon->priv->grabbed_keyboard);
-	}
+	GdkDisplay *display;
+	GdkSeat *seat;
+
+	display = gtk_widget_get_display (icon->priv->dock);
+	seat = gdk_display_get_default_seat (display);
+
+	gdk_seat_ungrab (seat);
+
 	gtk_grab_remove (icon->priv->dock);
 
 	/* hide again */
@@ -131,18 +126,20 @@ popup_dock (GvcStreamStatusIcon *icon,
         GdkRectangle   monitor;
         GtkRequisition dock_req;
         GdkWindow     *window;
-        GdkDeviceManager *device_manager;
-        GList         *list;
-        GList         *link;
-        gboolean       grabbed;
+        GdkSeat *seat;
+        GdkSeatCapabilities capabilities;
+        GdkGrabStatus status;
 
         update_dock (icon);
 
+        G_GNUC_BEGIN_IGNORE_DEPRECATIONS
         screen = gtk_status_icon_get_screen (GTK_STATUS_ICON (icon));
         res = gtk_status_icon_get_geometry (GTK_STATUS_ICON (icon),
                                             &screen,
                                             &area,
                                             &orientation);
+        G_GNUC_END_IGNORE_DEPRECATIONS
+
         if (! res) {
                 g_warning ("Unable to determine geometry of status icon");
                 return FALSE;
@@ -198,49 +195,15 @@ popup_dock (GvcStreamStatusIcon *icon,
 
         display = gtk_widget_get_display (icon->priv->dock);
         window = gtk_widget_get_window (icon->priv->dock);
-        device_manager = gdk_display_get_device_manager (display);
+        seat = gdk_display_get_default_seat (display);
 
-        grabbed = FALSE;
-        list = gdk_device_manager_list_devices (device_manager, GDK_DEVICE_TYPE_MASTER);
-        for (link = list; link != NULL; link = g_list_next (link)) {
-                GdkDevice *device = GDK_DEVICE (link->data);
-                if (gdk_device_get_source (device) == GDK_SOURCE_KEYBOARD)
-                        continue;
-                if (gdk_device_grab (device, window,
-                                     GDK_OWNERSHIP_NONE, TRUE,
-                                     GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
-                                     GDK_POINTER_MOTION_MASK | GDK_SCROLL_MASK,
-                                     NULL, time) != GDK_GRAB_SUCCESS) {
-                        icon->priv->grabbed_pointer = g_object_ref (device);
-                        grabbed = TRUE;
-                        break;
-                }
-        }
-        g_list_free (list);
+        capabilities = GDK_SEAT_CAPABILITY_POINTER |
+                       GDK_SEAT_CAPABILITY_KEYBOARD;
 
-        if (grabbed) {
-                ungrab (icon, time);
-                return FALSE;
-        }
+        status = gdk_seat_grab (seat, window, capabilities, TRUE, NULL,
+                                NULL, NULL, NULL);
 
-        grabbed = FALSE;
-        list = gdk_device_manager_list_devices (device_manager, GDK_DEVICE_TYPE_MASTER);
-        for (link = list; link != NULL; link = g_list_next (link)) {
-                GdkDevice *device = GDK_DEVICE (link->data);
-                if (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD)
-                        continue;
-                if (gdk_device_grab (device, window,
-                                     GDK_OWNERSHIP_NONE, TRUE,
-                                     GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK,
-                                     NULL, time) != GDK_GRAB_SUCCESS) {
-                        icon->priv->grabbed_keyboard = g_object_ref (device);
-                        grabbed = TRUE;
-                        break;
-                }
-        }
-        g_list_free (list);
-
-        if (grabbed) {
+        if (status != GDK_GRAB_SUCCESS) {
                 ungrab (icon, time);
                 return FALSE;
         }
@@ -331,6 +294,8 @@ on_status_icon_popup_menu (GtkStatusIcon       *status_icon,
         gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
         gtk_widget_show_all (menu);
+
+        G_GNUC_BEGIN_IGNORE_DEPRECATIONS
         gtk_menu_popup (GTK_MENU (menu),
                         NULL,
                         NULL,
@@ -338,6 +303,7 @@ on_status_icon_popup_menu (GtkStatusIcon       *status_icon,
                         status_icon,
                         button,
                         activate_time);
+        G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
 static gboolean
@@ -437,8 +403,11 @@ update_icon (GvcStreamStatusIcon *icon)
         /* apparently status icon will reset icon even if
          * if doesn't change */
         if (icon->priv->current_icon != n) {
+                G_GNUC_BEGIN_IGNORE_DEPRECATIONS
                 gtk_status_icon_set_from_icon_name (GTK_STATUS_ICON (icon),
                                                     icon->priv->icon_names [n]);
+                G_GNUC_END_IGNORE_DEPRECATIONS
+
                 icon->priv->current_icon = n;
         }
 
@@ -469,7 +438,11 @@ update_icon (GvcStreamStatusIcon *icon)
                                           100 * (float)volume / PA_VOLUME_NORM,
                                           gvc_mixer_stream_get_description (icon->priv->mixer_stream));
         }
+
+        G_GNUC_BEGIN_IGNORE_DEPRECATIONS
         gtk_status_icon_set_tooltip_markup (GTK_STATUS_ICON (icon), markup);
+        G_GNUC_END_IGNORE_DEPRECATIONS
+
         g_free (markup);
 }
 
@@ -543,6 +516,11 @@ gvc_stream_status_icon_set_mixer_stream (GvcStreamStatusIcon *icon,
 
                 g_object_ref (icon->priv->mixer_stream);
 
+                gvc_channel_bar_set_base_volume (GVC_CHANNEL_BAR (icon->priv->bar),
+                                                 gvc_mixer_stream_get_base_volume (stream));
+                gvc_channel_bar_set_is_amplified (GVC_CHANNEL_BAR (icon->priv->bar),
+                                                  gvc_mixer_stream_get_can_decibel (stream));
+
                 icon->priv->thaw = TRUE;
                 adj = GTK_ADJUSTMENT (gvc_channel_bar_get_adjustment (GVC_CHANNEL_BAR (icon->priv->bar)));
                 gtk_adjustment_set_value (adj,
@@ -557,6 +535,7 @@ gvc_stream_status_icon_set_mixer_stream (GvcStreamStatusIcon *icon,
                                   "notify::is-muted",
                                   G_CALLBACK (on_stream_is_muted_notify),
                                   icon);
+
         }
 
         update_icon (icon);
@@ -645,8 +624,10 @@ gvc_stream_status_icon_constructor (GType                  type,
 
         icon = GVC_STREAM_STATUS_ICON (object);
 
+        G_GNUC_BEGIN_IGNORE_DEPRECATIONS
         gtk_status_icon_set_from_icon_name (GTK_STATUS_ICON (icon),
                                             icon->priv->icon_names[0]);
+        G_GNUC_END_IGNORE_DEPRECATIONS
 
         /* window */
         icon->priv->dock = gtk_window_new (GTK_WINDOW_POPUP);
@@ -708,9 +689,6 @@ gvc_stream_status_icon_dispose (GObject *object)
                 icon->priv->mixer_stream = NULL;
         }
 
-        g_clear_object (&icon->priv->grabbed_pointer);
-        g_clear_object (&icon->priv->grabbed_keyboard);
-
         G_OBJECT_CLASS (gvc_stream_status_icon_parent_class)->dispose (object);
 }
 
@@ -786,8 +764,6 @@ gvc_stream_status_icon_init (GvcStreamStatusIcon *icon)
                           NULL);
 
         icon->priv->thaw = FALSE;
-        icon->priv->grabbed_pointer = NULL;
-        icon->priv->grabbed_keyboard = NULL;
 }
 
 static void
